﻿(function (app) {
    var overlayView,
        ratingElementSelector = "select.rating",
        tutorialRating;
		tutorialsDataPromise = app.tutorialsContent.get(),
        constants = {
            iconClassTemplate: "check-completed-",
            imageClassTemplate: "mountain-completed-",
            titleClassTemplate: "title-completed-"
        };

    app.overlays = {};

    kendo.data.binders.addClass = kendo.data.Binder.extend({
        refresh: function () {
            var that = this,
                value = that.bindings.addClass.get();

            $(that.element).addClass(value);
        }
    });

    app.overlays.load = function (overlayId, tutorialIndex, lessonIndex) {
        tutorialRating = null;

        tutorialsDataPromise.done(function (tutorialsData) {
            var viewModel = getViewModel(overlayId, tutorialsData, tutorialIndex, lessonIndex);

            $(ratingElementSelector).barrating("destroy");
            if (overlayView) {
                overlayView.destroy();
            }

            console.log(overlayId + "-template");
            overlayView = new kendo.View(overlayId + "-template", { model: viewModel, wrap: false });
            overlayView.render(".overlay");

            initRatingElement(tutorialsData[tutorialIndex], viewModel);

            app.lifeCycleBridge.raise("ready");
        });

        console.log("load overlay: ", arguments);
    };

    function getViewModel(overlayId, tutorialsData, tutorialIndex, lessonIndex) {
        var viewModel,
            tutorial = tutorialsData[tutorialIndex],
            lesson = tutorial.lessons[lessonIndex],
            nextLesson = tutorial.lessons[lessonIndex + 1],
            classVariation = lessonIndex === 0 ? "first" : (lessonIndex === tutorial.lessons.length ? "last" : "middle");

        if (nextLesson) {
            viewModel = {
                title: "You successfully completed Lesson " + lesson.displayNumber,
                continueLessonText: "Continue with Lesson " + nextLesson.displayNumber + ":",
                nextLessonTitle: nextLesson.title,
                steps: nextLesson.steps,
                imageClass: constants.imageClassTemplate + classVariation,
                iconClass: constants.iconClassTemplate + classVariation,
                titleClass: constants.titleClassTemplate + classVariation,
                okText: "Continue with Lesson " + nextLesson.displayNumber,
                cancelText: "Not now",
                clickYes: function () {
                    app.overlaysBridge.sendResponse(overlayId, tutorialIndex, lessonIndex, true);
                },
                clickNo: function () {
                    app.overlaysBridge.sendResponse(overlayId, tutorialIndex, lessonIndex, false);
                }
            };
        }
        else {
            viewModel = {
                title: "You completed the \"" + tutorial.title + "\" tutorial",
                ratingTitle: "Rate the \"" + tutorial.title + "\" tutorial",
                imageClass: constants.imageClassTemplate + classVariation,
                iconClass: constants.iconClassTemplate + classVariation,
                titleClass: constants.titleClassTemplate + classVariation,
                okText: "Close",
                clickYes: function () {
                    if (tutorialRating) {
                        // Analytics: TutorialCompleteRating
                        app.analyticsBridge.trackTutorialCompleteRating(tutorial.title, tutorial.type, tutorialRating);
                    }

                    app.overlaysBridge.sendResponse(overlayId, tutorialIndex, lessonIndex, false);
                }
            };
        }
        
        return kendo.observable(viewModel);
    }

    function initRatingElement(tutorial, viewModel) {
        ratingElement = $(ratingElementSelector).barrating({
            theme: "fontawesome-stars",
            initialRating: null,
            onSelect: function (value) {
                tutorialRating = value;

                viewModel.set("ratingTitle", "Thank you for your feedback.")
            }
        });
    }
})(window.app = window.app || {})