﻿(function (app) {
    var urlPartSeparator = app.constants.urlPartSeparator;

    app.routes = {
        root: "/",
        lessons: "/tutorial/:index",
        steps: "/tutorial/:index/:lessonIndex/:stepIndex",
        getRoute: function (route, params, queryParams) {
            var routeParts = route.substring(1).split(urlPartSeparator),
                paramIndex = 0;

            routeParts.forEach(function (item, index) {
                if (item[0] === ":") {
                    routeParts[index] = params[paramIndex++];
                }
            });

            route = urlPartSeparator + routeParts.join(urlPartSeparator);

            if (queryParams) {
                route += "?" + queryParams;
            }

            return route;
        }
    };

    app.router = new kendo.Router({
        init: function () {
            app.views.layout.render("body > .app");
        },
        change: function (e) {
            var match,
                mainSection = $("body > .app .main-section:first")[0];

            mainSection.scrollTop = 0;
            app.router.level = (match = e.url.match(/\//g)) ? match.length - 1 : 0;
        },
        level: 0
    });

    app.router.route(app.routes.root, function () {
        var title = app.models.tutorials.title,
            totalProgress = app.models.tutorials.totalProgress(),
            backgroundImage = app.ui.getProgressBasedBackgroundImage(totalProgress);


        app.ui.updateLayoutModel(title, totalProgress, backgroundImage);

        if (app.views.tutorials) {
            app.views.tutorials.destroy();
        }

        app.views.tutorials = app.kendoUtils.createNewView("tutorials-template", app.models.tutorials);

        app.views.layout.showIn("#view", app.views.tutorials);
    });

    app.router.route(app.routes.lessons, function (tutorialIndex) {
        var lessonsView = getOrAddLessonsView(tutorialIndex),
            tutorialProgress = lessonsView.model.progress(),
            backgroundImage = app.ui.getProgressBasedBackgroundImage(tutorialProgress);

        app.ui.updateLayoutModel(lessonsView.model.metadata.title, tutorialProgress, backgroundImage);

        app.views.layout.showIn("#view", lessonsView);
    });

    app.router.route(app.routes.steps, function (tutorialIndex, lessonIndex, stepIndex) {
        var tutorial = app.models.tutorials.items[tutorialIndex],
            lesson = tutorial.lessons[lessonIndex],
            lessonTitle = lesson.metadata.title,
            lessonProgress = lesson.progress(),
            stepView = getOrAddLessonStepView(tutorialIndex, lessonIndex, stepIndex),
            backgroundImage = app.ui.getProgressBasedBackgroundImage(lessonProgress);

        tutorial.set("currentOpenLessonIndex", lessonIndex);
        app.ui.updateLayoutModel(lessonTitle, lessonProgress, backgroundImage);

        app.views.layout.showIn("#view", stepView);
    });

    function getOrAddLessonsView(tutorialIndex) {
        if (app.views.lessonViews[tutorialIndex]) {
            app.views.lessonViews[tutorialIndex].destroy();
        }

        app.views.lessonViews[tutorialIndex] = app.kendoUtils.createNewView("lessons-template", app.models.tutorials.items[tutorialIndex]);

        return app.views.lessonViews[tutorialIndex];
    }

    function getOrAddLessonStepView(tutorialIndex, lessonIndex, stepIndex) {
        var lessonStepViewKey = [tutorialIndex, lessonIndex, stepIndex].join("_"),
            stepModel = app.models.tutorials.items[tutorialIndex].lessons[lessonIndex].steps[stepIndex];

        if (app.views.lessonStepViews[lessonStepViewKey]) {
            app.views.lessonStepViews[lessonStepViewKey].destroy();
        }

        app.views.lessonStepViews[lessonStepViewKey] = app.kendoUtils.createNewView("lesson-step-content-template", stepModel);

        return app.views.lessonStepViews[lessonStepViewKey];
    }
})(window.app = window.app || {});