﻿(function (app) {
    app.steps = {
        initialize: function (tutorial) {
            tutorial.lessons.forEach(function (lesson, index) {
                lesson.steps = lesson.metadata.steps
                    .map(function (item, index) {
                        return {
                            metadata: item,
                            tutorial: function () {
                                return this.lesson().tutorial();
                            },
                            lesson: function () {
                                return this.parent().parent();
                            },
                            tutorialIndex: tutorial.metadata.index,
                            lessonIndex: lesson.metadata.index,
                            title: function () {
                                var titleContainer = $("<div>"),
                                    fullContent = this.get("fullContent").call(this),
                                    titleElement = app.markdownUtils.getContentTitle(fullContent),
                                    stepCompletedBadge = $("<span>");

                                stepCompletedBadge.addClass("label");
                                if(this.get("isCompleted")) {
                                    stepCompletedBadge.text("Completed");
                                    stepCompletedBadge.addClass("label-success");
                                }
                                else {
                                    stepCompletedBadge.text("Not completed");
                                    stepCompletedBadge.addClass("label-default");
                                }

                                titleElement.append(stepCompletedBadge);
                                titleContainer.append(titleElement);
                                titleContainer.append($("<hr>"));

                                return titleContainer.html();
                            },
                            content: function () {
                                var fullContent = this.get("fullContent").call(this),
                                    contentContainerWithoutTitle = app.markdownUtils.removeContentTitle(fullContent);

                                return contentContainerWithoutTitle.html();
                            },
                            fullContent: function () {
                                return app.markdownUtils.convertToHtml(item.contentRaw);
                            },
                            isCompleted: false,
                            isPreviousStepVisible: index > 0 || lesson.metadata.index > 0,
                            isCompleteStepVisible: function () {
                                return !this.get("isCompleted");
                            },
                            isCompleteStepEnabled: function () {
                                var prevIndexes = getNextPrevIndexes.call(this, false),
                                    prevStep = prevIndexes.lessonIndex !== -1 &&
                                               app.models.tutorials.items[this.tutorialIndex].lessons[prevIndexes.lessonIndex].steps[prevIndexes.stepIndex];

                                return this.get("isCompleteStepVisible").call(this) && (!prevStep || prevStep.get("isCompleted"));
                            },
                            CompleteStepTooltip: function () {
                                return this.get("isCompleteStepEnabled").call(this) ? "" : "You must complete the previous step first";
                            },
                            isNextStepVisible: function () {
                                var isLastStep = index === lesson.steps.length -1,
                                    isLastLesson = lesson.metadata.index === tutorial.lessons.length - 1;

                                return !(isLastStep && isLastLesson);
                            },
                            onPreviousStepClick: onPreviousStepClick,
                            onCompleteStepClick: onCompleteStepClick,
                            onNextStepClick: onNextStepClick,
                            onStepClick: onStepClick
                        };
                    });

                delete lesson.metadata.steps;
            });
        }
    };

    function onPreviousStepClick(e) {
        e.stopPropagation();

        var prevIndexes = getNextPrevIndexes.call(this, false);
        this.tutorial().set("currentOpenLessonIndex", prevIndexes.lessonIndex);
        goToStep.call(this, prevIndexes.lessonIndex, prevIndexes.stepIndex);
    }

    function onCompleteStepClick(e) {
        var nextIndexes = getNextPrevIndexes.call(this, true),
            navigationCallback;

        e.stopPropagation();

        // Analytics: TutorialLessonStepComplete
        app.analyticsBridge.trackTutorialLessonStepComplete(
            this.tutorial().metadata.title, this.tutorial().metadata.type,
            this.lesson().metadata.displayNumber, this.lesson().metadata.title,
            this.metadata.displayNumber, this.metadata.title);

        // ProgressData: SetCompletedStep
        app.progressDataBridge.setCompletedStep(this.tutorialIndex, this.lessonIndex, this.metadata.index);

        this.set("isCompleted", true);

        app.ui.updateLayoutModel(
            this.lesson().metadata.title,
            this.lesson().progress(),
            app.ui.getProgressBasedBackgroundImage(this.lesson().progress()));

        if (nextIndexes.isLessonCompleted) {
            // Analytics: TutorialLessonComplete
            app.analyticsBridge.trackTutorialLessonComplete(
                this.tutorial().metadata.title, this.tutorial().metadata.type,
                this.lesson().metadata.displayNumber, this.lesson().metadata.title);
        }

        if (nextIndexes.isTutorialCompleted) {
            // Analytics: TutorialComplete
            app.analyticsBridge.trackTutorialComplete(this.tutorial().metadata.title, this.tutorial().metadata.type);
        }

        this.tutorial().set("currentOpenLessonIndex", nextIndexes.lessonIndex);

        navigationCallback = goToStep.bind(this, nextIndexes.lessonIndex, nextIndexes.stepIndex);
        showOverlayIfNecessary.call(this, nextIndexes, navigationCallback);
    }

    function onNextStepClick(e) {
        var nextIndexes = getNextPrevIndexes.call(this, true);

        e.stopPropagation();

        this.tutorial().set("currentOpenLessonIndex", nextIndexes.lessonIndex);
        goToStep.call(this, nextIndexes.lessonIndex, nextIndexes.stepIndex);
    }

    function onStepClick(e) {
        e.stopPropagation();

        goToStep.call(this, this.lessonIndex, this.metadata.index);
    }

    function showOverlayIfNecessary(nextPrevIndexes, navigationCallback) {
        var showOverlay = nextPrevIndexes.isLessonCompleted || nextPrevIndexes.isTutorialCompleted,
            overlayId, canceledNavigationRoute;

        if (showOverlay) {
            overlayId = nextPrevIndexes.isTutorialCompleted ? "tutorial-completed" : "lesson-completed";
            canceledNavigationRoute = nextPrevIndexes.isTutorialCompleted ? app.routes.root : app.routes.getRoute(app.routes.lessons, [this.tutorialIndex]);

            app.overlaysBridge.show(overlayId, this.tutorialIndex, this.lessonIndex)
                .done(function (shouldNavigate) {
                    if (!shouldNavigate) {
                        app.router.navigate(canceledNavigationRoute);
                        return;
                    }

                    navigationCallback();
                });
        }
        else {
            navigationCallback();
        }
    }

    function getNextPrevIndexes(isNext) {
        var stepIndex = this.metadata.index,
            lessonIndex = this.lessonIndex,
            lessons = app.models.tutorials.items[this.tutorialIndex].lessons,
            currentLesson = lessons[lessonIndex],
            isLessonCompleted = false,
            isTutorialCompleted = false;

        if (isNext) {
            stepIndex++;
            if (stepIndex === currentLesson.steps.length) {
                isLessonCompleted = true;
                lessonIndex++;
                stepIndex = 0;

                if (!lessons[lessonIndex]) {
                    isTutorialCompleted = true;
                }
            }
        }
        else {
            stepIndex--;
            if (stepIndex === -1) {
                lessonIndex--;
                if (lessons[lessonIndex]) {
                    stepIndex = lessons[lessonIndex].steps.length - 1;
                }
            }
        }

        if (!lessons[lessonIndex]) {
            lessonIndex = -1;
        }

        return {
            isLessonCompleted: isLessonCompleted,
            isTutorialCompleted: isTutorialCompleted,
            lessonIndex: lessonIndex,
            stepIndex: stepIndex,
        }
    }

    function goToStep(lessonIndex, stepIndex) {
        var route;

        if (lessonIndex !== -1) {
            route = app.routes.getRoute(app.routes.steps, [this.tutorialIndex, lessonIndex, stepIndex]);
        }
        else {
            route = app.routes.root;
        }

        app.router.navigate(route);
    }
})(window.app = window.app || {})