(function (app) {
    var progressDataPromise = app.progressDataBridge.get();

    app.ui = {
        createViewModels: function () {
            var deferred = $.Deferred();

            app.models = {};
            app.models.layoutModel = app.layout.getModel();

            app.tutorialsContent.get().done(function (tutorialsData) {
                app.models.tutorials = app.tutorials.getTutorialsModel(tutorialsData);
                setupCurrentTutorialsProgress(app.models.tutorials).done(deferred.resolve);
            }).fail(app.onDeferredFail);

            return [deferred];
        },
        createViewsAndLayouts: function () {
            app.views = {
                layout: new kendo.Layout("layout-template", { model: app.models.layoutModel, wrap: false }),
                tutorials: null,
                lessonViews: [],
                lessonStepViews: []
            };
        },
        updateLayoutModel: function (title, progress, progressBackgroundImage) {
            app.layout.setModelVars(app.models.layoutModel, title, progress, progressBackgroundImage);
        },
        getProgressBasedBackgroundImage: function (progress) {
            var deg, color;


            deg = progress * 3.6;
            if (deg <= 180) {
                deg = 90 + deg;
                color = "dee5e6";
            }
            else {
                deg = deg - 90;
                color = "3497da";
            }

            return "linear-gradient(" + deg + "deg, transparent 50%, #" + color + " 50%), linear-gradient(90deg, #dee5e6 50%, transparent 50%)";
        }
    };

    function setupCurrentTutorialsProgress(tutorialsModel) {
        var deferred = $.Deferred();

        if (!app.isInsideVSE) {
            deferred.resolve();
            return deferred;
        }

        progressDataPromise.done(function (data) {
            var tutorialFileNames = Object.keys(data),
                tutorialIndexes = [];

            tutorialsModel.items.forEach(function (tutorial, tutorialIndex) {
                var tutorialFileNameIndex = tutorialFileNames.indexOf(tutorial.metadata.fileName);
                if (tutorialFileNameIndex !== -1) {
                    data[tutorialIndex] = data[tutorialFileNames[tutorialFileNameIndex]];
                    tutorialIndexes.push(tutorialIndex);
                }
            });

            tutorialIndexes.forEach(function (tutorialIndex) {
                var tutorial = tutorialsModel.items[tutorialIndex],
                    lessonStepData = data[tutorialIndex],
                    lessonStepDataParts, completedLessonIndex, completedStepIndex;

                tutorial.set("isStarted", true);

                if (lessonStepData !== "") {
                    lessonStepDataParts = lessonStepData.split("_");
                    completedLessonIndex = lessonStepDataParts[0] - 0;
                    completedStepIndex = lessonStepDataParts[1] - 0;

                    tutorial.lessons.every(function (lesson, lessonIndex) {
                        if (lessonIndex <= completedLessonIndex) {
                            lesson.steps.every(function (step, stepIndex) {
                                if (lessonIndex < completedLessonIndex || stepIndex <= completedStepIndex) {
                                    step.set("isCompleted", true);
                                    return true;
                                }
                            });
                            return true;
                        }
                    });
                }
            });

            deferred.resolve();
        });

        return deferred;
    }
})(window.app = window.app || {});