﻿(function (app) {
    var requestedBridges = {},
        emptyFunction = function () { },
        bridgesMethodNames,
        bridgesDummy;

    app.isInsideVSE = !!window.cefQuery;

    app.bridgeHelper = {
        execute: function (bridgeName, action) {
            getBridge(bridgeName).done(action);
        },
        cefQuery: function (bridgeName, methodName, args, failResult, showLoading) {
            var deferred = $.Deferred(),
                methodFullName = bridgeName + "." + methodName;

            args = args || {};

            if (showLoading) {
                app.loadingUtils.showLoading();
            }

            this.execute(bridgeName, function (projectHelperBridge) {
                window.cefQuery({
                    request: methodFullName,
                    args: args,
                    onSuccess: function (response) {
                        deferred.resolve(response);
                        console.log(methodFullName + ":", response);

                        if (showLoading) {
                            app.loadingUtils.hideLoading();
                        }
                    },
                    onFailure: function () {
                        console.log(arguments);
                        deferred.resolve(failResult);
                    }
                });
            });

            return deferred;
        }
    };

    function getBridge(bridgeName) {
        var deferred;

        if (requestedBridges[bridgeName]) {
            return requestedBridges[bridgeName];
        }

        deferred = $.Deferred();
        requestedBridges[bridgeName] = deferred;

        (function init() {
            if (!app.isInsideVSE) {
                deferred.resolve(bridgesDummy);
                return;
            }

            if (!window[bridgeName]) {
                window.setTimeout(init);
                return;
            }

            deferred.resolve(window[bridgeName]);
        })();

        return deferred;
    }


    if (!app.isInsideVSE) {
        // Polyfill bridges to make app work on its own
        bridgesMethodNames = [
            "TrackFeatureUsage",
            "GetProgressData",
            "SetCompletedStep",
            "AddTutorialData",
            "RemoveTutorialData",
            "OpenProject",
            "CreateProject",
            "CheckIfProjectExists",
            "IsSimulatorOpenForProject",
            "CloseSimulatorProcess",
            "GetSolutionPath",
            "OpenUrl",
            "Raise",
            "Show",
            "SendResponse",
            "ShowWarningMessage",
            "ShowErrorMessage"
        ]

        window.cefQuery = function (options) {
            options.onSuccess = options.onSuccess || emptyFunction;

            switch (true) {
                case options.request.endsWith("OpenProject"):
                    options.onSuccess(false)
                    break;
                default:
                    options.onSuccess("{}");
                    break;
            }
        };

        bridgesDummy = {};
        bridgesMethodNames.forEach(function (methodName) {
            bridgesDummy[methodName] = emptyFunction;
        });
    }
})(window.app = window.app || {});