﻿(function (app) {
    app.markdownUtils = {
        convertToHtml: convert,
        getContentTitle: getTitle,
        removeContentTitle: removeTitle
    };

    var actionWrapperOpen = '<div class="action">',
        actionWrapperClose = "</div>";

    function convert(markdown) {
        var html = marked(markdown),
            container = $("<div/>");

        html = formatActionSections(html);

        container.html(html);

        formatLinks(container);
        addSyntaxHighlighting(container);

        return container;
    }

    function formatActionSections(html) {
        // Match all <hr data-action="start" /> tags and replace them with <div class="action">.
        // Regex is flexible enough to match hr tags:
        // 1. With more than one space between "hr" and "data" words
        // 2. With data-action attribute with single or double quotes
        // 3. With zero or more spaces between data-action="start" and closing slash
        // 4. With optional closing slash
        html = html.replace(/<hr(\s)+data-action=[\"\']start[\"\'](\s)*\/?>/g, actionWrapperOpen);
        html = html.replace(/<hr(\s)+data-action=[\"\']end[\"\'](\s)*\/?>/g, actionWrapperClose);
        return html;
    }

    function formatLinks(container) {
        container.find("a")
            .attr({
                "onclick": function (index, value) {
                    return "app.browserBridge.openUrl('" + this.href + "'); return false;";
                },
                "style": "cursor: pointer;"
            })
            .removeAttr("target")
            .removeAttr("href");
    }

    function addSyntaxHighlighting(container) {
        container.find("code").each(function (i, block) {
            hljs.highlightBlock(block);
        });
    }

    function removeTitle(container) {
        getTitle(container).remove();

        return container;
    }

    function getTitle(container) {
        return container.find("h3:first");
    }
})(window.app = window.app || {});