﻿(function (app) {
    app.projectUtils = {
        openProject: function (tutorial, restart) {
            var projectMetadata = getProjectMetadata(tutorial),
                deferred = $.Deferred();

            app.projectHelperBridge
                .checkIfProjectExists(projectMetadata.projectName)
                .done(function (projectExists) {
                    var isNewProject = !projectExists || restart;

                    if (isNewProject) {
                        createNewProject(projectMetadata, deferred);
                    }
                    else {
                        app.projectHelperBridge.openProject(projectMetadata.projectName)
                            .done(deferred.resolve.bind(deferred, isNewProject));
                    }
                });

            return deferred;
        }
    };

    function createNewProject(projectMetadata, deferred) {
        var cancelCreation = deferred.reject,
            createProject = createNewProjectBridgeCallback.bind(null, projectMetadata, deferred);

        app.projectHelperBridge.isSimulatorOpenForProject(projectMetadata.projectName)
            .done(function (isSimulatorOpen) {
                if (!isSimulatorOpen) {
                    createProject();
                    return;
                }

                app.messageBoxBridge.showWarning(
                    "The Device Simulator is locking the \"" + projectMetadata.projectName + "\" project directory. Do you want to close the simulator now?",
                    "\"" + projectMetadata.projectName + "\" project directory is locked")
                        .done(function (response) {
                            if (response) {
                                app.projectHelperBridge.closeSimulator(projectMetadata.projectName)
                                    .done(createProject);
                            }
                            else {
                                cancelCreation();
                            }
                        });
            });
    }

    function createNewProjectBridgeCallback(projectMetadata, deferred) {
        app.projectHelperBridge.createProject(projectMetadata)
            .done(function (isCreated) {
                if (isCreated) {
                    deferred.resolve(true);
                }
                else {
                    deferred.reject();
                    showProjectDirectoryLockedError(projectMetadata.projectName);
                }
            });
    }

    function showProjectDirectoryLockedError(projectName) {
        app.projectHelperBridge.getSolutionPath(projectName)
            .done(function (solutionPath) {
                app.messageBoxBridge.showError(
                        "The \"" + projectName + "\" project directory (" + solutionPath + ") is locked by an external application. Before you can continue, you need to close the external application.",
                        "\"" + projectName + "\" project directory is locked");
            });
    }

    function getProjectMetadata(tutorial) {
        return {
            tutorialName: tutorial.metadata.fileName,
            tutorialType: tutorial.metadata.type === "hybrid" ? "Cordova" : "NativeScript",
            projectName: tutorial.metadata.app.projectName
        };
    }
})(window.app = window.app || {});